// $Id$
/** *****************************************************************************************
@doxygen

@file tsonewin.h

@brief Abfragen von Dallas DS1820 One-Wire-Thermometer.

Siehe auch <a href=../../../deutsch/lq/tsonewin.htm>../../../deutsch/lq/tsonewin.htm</a>
<br>
<a href=../../../../../etronik/sensor/DS18S20.pdf>../../../../../etronik/sensor/DS18S20.pdf</a>

@author HJHerbert

@version 2014-08-16 - noch ohne CRC-Test
@version 2015-10-28 - mit CRC8-Tst
@version 2015-11-06 - Alle anstarten, warten, dann alle auslesen., alle 8 Kanaele mit 1 Hz
@version 2015-11-24 - Interpolation auch bei negativen Temperaturen korrekt.
@version 2016-11-27 - Anzahl jetzt berechnet


********************************************************************************************* */
#ifndef __tsonewin_h__
#define __tsonewin_h__
#define SERNUMLEN 8

#ifndef __heimath_h__
#include <heimath.h>
#endif




#if defined( __TURBOC__ ) || defined ( _MSC_VER )
// Unter DOS / Win95 das printerport verwenden
//#define LPT		// LPT-Port verwenden
#endif

#ifdef AVR

// Im makefile stand EINE der Definitionen:
//                 PORTA                     PORTB                     PORTC                     PORTD
// Bit 0 -DTSONEWI_PORTA=0xFF, oder -DTSONEWI_PORTB=0xFF, oder -DTSONEWI_PORTC=0xFF, oder -DTSONEWI_PORTD=0xFF
#if defined(TSONEWI_PORTA)
// Es wird AVR PORTA verwendet
#define TSONEWI_PIN PINA
#define TSONEWI_PORT PORTA
#define TSONEWI_DDR DDRA
#define TSONEWI_MASKE TSONEWI_PORTA

#elif defined(TSONEWI_PORTB)
// Es wird AVR PORTB verwendet
#define TSONEWI_PIN PINB
#define TSONEWI_PORT PORTB
#define TSONEWI_DDR DDRB
#define TSONEWI_MASKE TSONEWI_PORTB

#elif defined(TSONEWI_PORTC)
// Es wird AVR PORTC verwendet
#define TSONEWI_PIN PINC
#define TSONEWI_PORT PORTC
#define TSONEWI_DDR DDRC
#define TSONEWI_MASKE TSONEWI_PORTC

#elif defined(TSONEWI_PORTD)
// Es wird AVR PORTD verwendet
#define TSONEWI_PIN PIND
#define TSONEWI_PORT PORTD
#define TSONEWI_DDR DDRD
#define TSONEWI_MASKE TSONEWI_PORTD

#elif defined(TSONEWI_PORTE)
// Es wird AVR PORTE verwendet
#define TSONEWI_PPIN PINE
#define TSONEWI_PORT PORTE
#define TSONEWI_DDR DDRE
#define TSONEWI_MASKE TSONEWI_PORTE

#endif


#define TSONEWI_BIT bit // Das Bit aus der Struktur

// #define TSONEWI_BIT 0x40		// 0x40 = PD6  
#define SETZE0 TSONEWI_PORT &= ~TSONEWI_BIT ;	/* 0V ausgeben */ \
		TSONEWI_DDR |= TSONEWI_BIT & TSONEWI_MASKE ;			/* Aktiv */

#define SETZE1 TSONEWI_PORT |= TSONEWI_BIT ;		/* 5V ausgeben */ \
		TSONEWI_DDR |= TSONEWI_BIT & TSONEWI_MASKE ;				/* Aktiv */

#define PASSIV TSONEWI_DDR &= ~TSONEWI_BIT ;		/* Nicht Aktiv */ \
		TSONEWI_PORT &= ~TSONEWI_BIT ;		/* nicht mit Pull-Up, der ist extern */

#define EINGANG ((TSONEWI_PIN & TSONEWI_BIT)!=0)

#else		// #ifdef AVR


#define TSONEWI_MASKE 0x30
// DOS; WIN95
#ifdef LPT        // PCPARTWI
u8 lpt ;
Das geht nicht wirklich, schon wegen der TTL-Pegel
#define TSONEWI_MASKE 0xF8 - Bit 0,1,2 haben keinen Eingang
#define OMASK bit
#define SETZE0 lpt |=  OMASK ;	/* 0V ausgeben: Transistor ansteuern */ \
		outportb( 0x378, lpt );
#define SETZE1 lpt &= ~OMASK ;	/* 5V ausgeben: Transistor nicht ansteuern */ \
		outportb( 0x378, lpt );
#define PASSIV lpt &= ~OMASK ;	/* 5V ausgeben, der Ausgang ist so aehnlich wie ein offener Kollektor */ \
		outportb( 0x378, lpt );
		/* Siehe hhierzu: file:///I:/h/home/html/hjh/etronik/s/pcpar.htm */
#define EINGANG ( ( bit & 0x80 ) ? ( !inportb( 0x379 ) & bit ) : ( inportb( 0x379 ) & bit )

#else
// Linux, Windows
#define SETZE0
#define SETZE1
#define PASSIV
#define EINGANG 0
#endif	// LPT
#endif // AVR

/*!
 * Feststellen, wie viele Bits im Port
 */

// @version 2016-11-27 - Anzahl jetzt berechnet


#define TSONEWI_BITZAHL ( \
			((TSONEWI_MASKE&0x01)?1:0) + \
			((TSONEWI_MASKE&0x02)?1:0) + \
			((TSONEWI_MASKE&0x04)?1:0) + \
			((TSONEWI_MASKE&0x08)?1:0) + \
			((TSONEWI_MASKE&0x10)?1:0) + \
			((TSONEWI_MASKE&0x20)?1:0) + \
			((TSONEWI_MASKE&0x40)?1:0) + \
			((TSONEWI_MASKE&0x80)?1:0) )


/*!
 * Diese Struktur wird TSONEWI_BITZAHL mal angelegt im Hauptprogramm, fuer jeden Aufnehmer ein Mal
 */


class CTSOneWireMehr 
	{
private:
//	u8 phase ;
public:
	u8 bit ;			// Leitungsadresse an PORTA
	u8 bu[9];			// Wurde ausgelesen: Messwert usw 8 Bytes plus CRC8
	u8 bus[8];			// Wurde ausgelesen: Seriennummer: 7 Bytes + CRC8
	// PGM_P fehler ;			// Fehler - String. Zeigt auf Programmspeicher
	u8		Fehlernummer ;		// Feste Zuordnung - siehe D:\h\home\cc\lq\tsonewin.cpp
	u8		Endfehlernummer ;	// Nach versuchter oder erfolgreicher Wandlung.
	s16		Temperaturwert;		// 1/100 Grad C
	u32		tick;				// Wann wurde gemessen ? gettick
	CRC8	crc8;
private:
	void Reset(void);			// Fehlerkennung fehler=0, Elektronisches anstarten
	void Schreibe0(void);		// Schreibe ein einzelnes 0-Bit
	void Schreibe1(void);		// Schreibe ein einzelnes 1-Bit
	u8 Lese(void);				// Hole 
	u8 Hole8Bits(void);
	void HoleNBytes(u8 *p,u8 n);
	void Sende8Bits(u8 b);
	void Sende8Bytes(u8 *p);
public:
	void HoleROM(void);				// Hole alle 9 Bytes nach -> bus[]
	void StarteMessung(void);		// Starte eine Messung: Sende Reset, dann 0xCC,0x44
	void HoleMesswert(void);		// Hole immer wieder einen ->Temperaturwert
	
	// Static-Funktionen fuer alles
	} ;



// Bis zu 8 Mal:
extern CTSOneWireMehr vCTSOneWireMehr[TSONEWI_BITZAHL] ;

	void CTSOneWireMehr__fun_one_wire(void) ;



#endif // __tsonewin_h__
