// $Id$
/** *****************************************************************************************
@doxygen

@file tsonewin.cpp

@brief Abfragen von Dallas DS1820 One-Wire-Thermometer.

Siehe auch <a href=../../../deutsch/lq/tsonewin.htm>../../../deutsch/lq/tsonewin.htm</a>
<br>
<a href=../../../../../etronik/sensor/DS18S20.pdf>../../../../../etronik/sensor/DS18S20.pdf</a>

@author HJHerbert

@version 2014-08-16 - noch ohne CRC-Test
@version 2015-10-28 - mit CRC8-Tst
@version 2015-11-06 - Alle anstarten, warten, dann alle auslesen., alle 8 Kanaele mit 1 Hz
@version 2015-11-24 - Interpolation auch bei negativen Temperaturen korrekt.

********************************************************************************************* */
#include <heport.h>
#include <helq.h>	// outportb()
#include <hegettim.h>	// delaytick()
#include <tshar.h>
#include <tsonewin.h>


#ifdef AVR
#if AVR_TIMER_DIVIDER_CODE !=2
#error One-Wire-Thnermometer DS18S20 brauchen Taktteiler 2, 3 geht nicht
#endif
#endif
/*
<br>ROM Kommandos sind:
<br>F0 - Suche ROM : Muss fuer jeden Slave wiederholt werden.
<br>33 - Lese ROM : Den ROM-Inhalt des einzigen Slave auslesen: 8 Bytes einlesen
<br>55 - ROM Vergleichen : 8 Bytes folgen
<br>CC - den einzigen Slave ansprechen
<br>EC - Grenzwertueberschreitungen suchen.
*/


/*!
 * Hier werden die 8 Klassen angelegt
 */
CTSOneWireMehr vCTSOneWireMehr[TSONEWI_BITZAHL] ;


/*!
 * Loese einen elektronischen Reset aus, initialisiere den DS1820 und erkenne die Rueckmeldung.
 * <br>Ergebnis: 
 * <ul><li>Fehlernummer = 0 - es ist kein Fehler aufgetreten.</li>
 * <li>Fehlernummer != 0 - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 */
void CTSOneWireMehr::Reset(void)
{
u32 starttick ;


SETZE1 ;

tsdelay( sectotick( 0.002000 ) );	// 

SETZE0 ;
starttick = gettick();
for(;;)
	{
	if ( EINGANG )
 		{				// Nicht Low
		Fehlernummer = 1 ; // PSTR("Reset(): Ich kann die Leitung nicht nach Low ziehen") ;
		return ;
		}
	if ( difftick( starttick, gettick() ) >= sectotick( 0.000480 ) ) 
		{				// lange genug fuer RESET
		break ; 
		}
	}

PASSIV ;

starttick = gettick();
for(;;)
	{
	if ( !EINGANG ) break ;	// Low
	if ( difftick( starttick, gettick() ) >= sectotick( 0.000480 ) )
		{
		Fehlernummer = 2 ; // PSTR("Reset(): Leitung wird nicht Low gehalten") ;
		return ;
		}
	}
// 18DS20 hat 0 geschickt
for(;;)
	{
	if ( EINGANG ) break ;
	if ( difftick( starttick, gettick() ) >= sectotick( 0.000880 ) )
		{
		Fehlernummer = 3 ; // = PSTR("Reset(): Leitung am Ende bleibt Low") ;
		return ;
		}
	}
// 18DS20 hat wieder 1 geschickt
//	reset_dt1=59 Ticks 0.000029 s
//  reset_dt2=289 Ticks 0.000144 s
// INIT-Impuls wurde beantwortet
delaytick( sectotick( 0.000008 ) );

Fehlernummer = 0 ; 		// kein Fehler bei Reset()
// 0.000008 : CC 64 bringt FE, sonst nichts
// 0.000520 : CC 64 bringt FE, sonst nichts
return ;
}


/*!
 * Schreibe ein 0-Bit
 */
void CTSOneWireMehr::Schreibe0(void) // > 60 + 5 us
{
SETZE0 ;
tsdelay( sectotick( 0.000115 ) );		// 60 ... 120
SETZE1 ;
tsdelay( sectotick( 0.000015 ) );	
}


/*!
 * Schreibe ein 1-Bit
 */
void CTSOneWireMehr::Schreibe1(void) // 10 + 55 us
{
SETZE0 ;

delaytick( sectotick( 0.000015 ) );	// 1 .. 15 us
SETZE1 ;
tsdelay( sectotick( 0.000115 ) );	// 60-t0 ... unbegrenzt
}



/*!
 * Lies ein Bit vom DS1820 ein.
 *
 * <br>Ergebnis: 
 * <ul><li>( TSOneWireMehr::fehler = 0 ) - es ist kein Fehler aufgetreten.</li>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * A : Probleme bei Kaelte: fehler 77 / 137
 * B : Probleme auch bei 19 grd C: fehler 77 / 137
 * C : Probleme auch bei 19 grd C: fehler 77 / 137
 
 * @return - das eingelesene Bit, 0 oder 1. Im Fehlerfall ist da 2 oder 3.
 */
FUNCT u8 CTSOneWireMehr::Lese(void)
{
u32 starttick ;

// A delaytick( sectotick( 0.000008 ) );	// 1...unendlich
// B delaytick( sectotick( 0.000008 ) );	// 1...unendlich
// C delaytick( sectotick( 0.000008 ) );	// 1...unendlich
delaytick( sectotick( 0.000035 ) );	// 1...unendlich

SETZE0 ;		// Sendeaufforderung
// A delaytick( sectotick( 0.000002 ) );	// 1  us
// B delaytick( sectotick( 0.000005 ) );	// 1  us
// C delaytick( sectotick( 0.0000000 ) );	// 1  us
delaytick( sectotick( 0.000001 ) );	// 1  us

PASSIV ;
// A delaytick( sectotick( 0.000003 ) );	// 15...60 us
// B delaytick( sectotick( 0.000010 ) );	// 15...60 us
// C // delaytick( sectotick( 0.000001 ) );	// 15...60 us
delaytick( sectotick( 0.000001 ) );	// 15...60 us


starttick = gettick();

if ( EINGANG )
	{	// Eine 1 steht an
	for(;;)
		{
		if ( !EINGANG ) // Fehler
			{
			Fehlernummer = 4 ; // PSTR("Lese(): anstehende 1 nicht stabil");
			return 3 ;
			}
		if ( difftick( starttick, gettick() ) >= sectotick( 0.000080 ) )
			{	// Das ist kein Fehler !!
			return 1 ;	// Die 1 stand stabil an
			}
		}
	}
else
	{
	for(;;)
		{
		if ( EINGANG ) // Fehler
			{	// Gesehen bei CC 64
			Fehlernummer = 5 ; // PSTR("Lese(): anstehende 0 nicht stabil");
			return 2 ;
			}
		if ( difftick( starttick, gettick() ) >= sectotick( 0.000005 ) )
			{
			break ;	// Die 0 stand stabil an
			}
		}
// Und jetzt warten, bis die 0 weggeht
	for(;;)
		{
		if ( EINGANG ) // Die ankommende 0 ist weggegangen
			{
			return 0 ;
			}
		if ( difftick( starttick, gettick() ) >= sectotick( 0.000090 ) )
			{			// Wartezeit ist angelaufen
			Fehlernummer = 6 ; // PSTR("Lese(): anstehende 0 geht nicht weg");
			return 4 ;	// Die 0 geht nicht mehr weg
			}
		}
		
	}
}



/*!
 * Lies ein Byte vom DS1820 ein.
 *
 * <br>Ergebnis: 
 * <ul><li>( TSOneWireMehr::fehler = 0 ) - es ist kein Fehler aufgetreten.</li>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * @return - das eingelesene Byte.
 */
FUNCT u8 CTSOneWireMehr::Hole8Bits(void)
{
u8 b ;
u8 i ;
b = 0 ;
for ( i = 8+1 ; --i ; )
	{	// i = 8,7,6,5,4,3,2,1
	switch( Lese() )
		{
	case 0 : b >>= 1 ; break ;
	case 1 : b >>= 1 ; b |= 0x80 ; break ;
	default :  ; // hat fehler gesetzt
		}
	tsnext("hole");
	}
return b ;
}




/*!
 * Sende ein Byte an den DS1820 ein.
 *
 * <br>Ergebnis: 
 * <ul><li>( TSOneWireMehr::fehler = 0 ) - es ist kein Fehler aufgetreten.</li>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * @param b - Das zu sendende Byte.
 */
FUNCT void CTSOneWireMehr::Sende8Bits(u8 b)
{
u8 i ;

delaytick( sectotick( 0.000160 ) );	// 

for ( i = 8+1 ; --i ; )
	{	// i = 8,7,6,5,4,3,2,1
	if ( b & 0x01 ) 
		{
		Schreibe1() ;
		}
	else
		{
		Schreibe0() ;
		}
	b >>= 1 ;
	tsnext("sende");
	}
// i = 0
return ;
}



/*!
 * Sende 8 Bytes an den DS1820 ein.
 *
 * <br>Ergebnis: 
 * <ul><li>( TSOneWireMehr::fehler = 0 ) - es ist kein Fehler aufgetreten.</li>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * @param p - Zeigt auf 8 Bytes.
 */
FUNCT void CTSOneWireMehr::Sende8Bytes(u8 *p)
{
u8 i ;

for ( i = 0 ; i<8 ; ++i)
	{
	Sende8Bits( p[i] );
	if ( Fehlernummer ) break ;
	}
// i = 0
return ;
}


/*!
 * Hole 8 Bytes vom DS1820 herein.
 *
 * <br>Ergebnis: 
 * <ul><li>( TSOneWireMehr::fehler = 0 ) - es ist kein Fehler aufgetreten.</li>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * Das Ergebnis ist in TSOneWireMehr::bu[0] ... TSOneWireMehr::bu[7]
 */
FUNCT void CTSOneWireMehr::HoleNBytes(u8 *p,u8 n)
{
u8 i ;

for ( i = 0 ; i < n ; ++i )
	{
	p[i] = Hole8Bits();	// Lese noch eine Zahl
	if ( Fehlernummer ) return ;	// Bit-Time-Out
	}

for ( i = 1 ; i < n ; ++i )
	{
	if ( p[i] != 0xff ) break ;	// Gute Werte
	}
if ( i == n )
	{	// Alle Werte = 0xff
	Fehlernummer = 7 ; // PSTR("Hole8Bytes(): nur ff gekommen");
	return ;
	}
// Pruefe CRC8
crc8.Start(0);
for ( i = 0 ; i < n-1 ; ++i )
	{
	crc8.NochEin(p[i]);
	}
if ( p[n-1] != crc8.Ergebnis() )
	{
	Fehlernummer = 8 ; // PSTR("Hole8Bytes(): CRC8-Fehler");
	return ;
	}
// Kein Fehler
return ;
}



/*!
 * Hole eine Temperatur von einen DS1820-Sensor.
 *
 * <br>Ergebnis: 
 * <ul>
 * <li>( TSOneWireMehr::fehler = text ) - es ist ein Fehler aufgetreten. Ansteuerung muss abgebrochen werden.</li>
 * </ul>
 * @param _bit - Die Bitmaske des Anschlusses: 1. Sensor:0x01; dann 0x02, 0x04, 0x08 .. ;8. Sensor:0x80
 */
FUNCT void CTSOneWireMehr::HoleROM()
{

// Dann holen wir die Seriennummer
Reset();
if ( Fehlernummer ) { Fehlernummer += 110 ; goto BeiFehler ; } 
Sende8Bits( 0x33 );		// Lese ROM
if ( Fehlernummer ) { Fehlernummer += 120 ; goto BeiFehler ; } 
HoleNBytes(&bus[0],8);		// Lese alle 7 Bytes plus CRC8 nach bus[] -> Seriennummer
if ( Fehlernummer ) { Fehlernummer += 130 ; goto BeiFehler; } 

return ;

BeiFehler:
	Endfehlernummer = Fehlernummer ;	// z.B: für MODBUS	

}




FUNCT void CTSOneWireMehr::StarteMessung()
{
Fehlernummer = 0 ;
Reset();
if ( Fehlernummer ) { Fehlernummer += 10 ; goto BeiFehler ; } 
Sende8Bits( 0xCC );		// Skip ROM
if ( Fehlernummer ) { Fehlernummer += 20 ; goto BeiFehler ; } 
Sende8Bits( 0x44 );	// Wandlung anstarten [Convert T]
if ( Fehlernummer ) { Fehlernummer += 30 ; goto BeiFehler ; } 

return ;

BeiFehler:
	Endfehlernummer = Fehlernummer ;	// z.B: für MODBUS	

}



FUNCT void CTSOneWireMehr::HoleMesswert()
{
s8 ganzegrade ;
Fehlernummer = 0 ;
Reset();
if ( Fehlernummer ) { Fehlernummer += 40 ; goto BeiFehler ; } 
Sende8Bits( 0xCC );		// Skip ROM
if ( Fehlernummer ) { Fehlernummer += 50 ; goto BeiFehler; } 
Sende8Bits( 0xBE );	// Lese alle 8 Zellen
if ( Fehlernummer ) { Fehlernummer += 60 ; goto BeiFehler; } 
HoleNBytes(&bu[0],9);		// Lese alle 8 Bytes plus CRC8
if ( Fehlernummer ) { Fehlernummer += 70 ; goto BeiFehler; } 
// Jetzt sollte der Temperaturwert sein in bu[0] und bu[1]


Endfehlernummer = 0 ;
tick = gettick() ;

// Umrechnen auf 1/100 grd C

/* Ohne Genauigkeitserweiterung, also Aufloesung = 0.5 grd C
Temperaturwert = *(s16 *)(&bu[0]) * 50 ; // interpretiere bu[0] und bu[1] als 16-Bit-Zahl, vorzeichenbehaftet.
*/

/* So wird - falsch berechnet
s16 teil100;

ganzegrade = *(s16 *)(&bu[0]) / 2 ;
teil100 = 100-mul_uc_uc(100,bu[6])/bu[7];
Temperaturwert = ganzegrade * 100 - 25 + teil100 ;
*/
/* So wird - falsch berechnet
ganzegrade = *(s16 *)(&bu[0]) / 2 ;

Temperaturwert = ((s16)ganzegrade * bu[7] + bu[7]-bu[6])*100/bu[7] ;
*/

/* so auch nicht
ganzegrade = *(s16 *)(&bu[0]) / 2 ;

Temperaturwert = (((s16)(ganzegrade+1) * bu[7] -bu[6])*(s16)100)/bu[7] ;
*/
/* So nur bis -20.49 grdC
ganzegrade = (*(s16 *)(&bu[0])) >>1 ;	// Das ist etwas anderes als /2 !!!

Temperaturwert = (((s16)(ganzegrade+1) * (s16)bu[7] -(s16)bu[6])*(s16)100-25)/(s16)bu[7] ;

// s16 g = ganzegrade+1 ;
// g *= bu[7];
// g -= (s16)bu[6] ;
// g *= 100 ;
// 21*1600=33600 ! Ueberlauf
// 20*1600=32000
*/

// So auch unter -20.49 grdC: Getestet bis -22 grdC (Tiefkühltruhe)
ganzegrade = (*(s16 *)(&bu[0])) >>1 ;	// Das ist etwas anderes als /2 !!!
Temperaturwert = div_sl_sw( mul_sw_sw( (ganzegrade+1)*bu[7]-bu[6], 100 ) , bu[7] ) -25 ;


// 0.0 grdC = 0
// 0.5 grdC = 1
return ;

BeiFehler:
	Endfehlernummer = Fehlernummer ;	// z.B: für MODBUS	

}



/*! Das wird 1 mal aufgerufen
 * Das ist eine tshar - Task
 * Die muss eingerichtet werden in main()
 * 
 * <br><code>tseinri( 1 , fun_mess, "fun_mess", stkDS1820, NSTK1820 );</code><br
 * Siehe sf/modone_n.cpp
 */
FUNCT void CTSOneWireMehr__fun_one_wire(void)
{
u8 romkanal ;

u8 i ;
u8 b ;
CTSOneWireMehr* pCTSOneWireMehr;


pCTSOneWireMehr = vCTSOneWireMehr ;
i = 0 ;

b = (TSONEWI_MASKE&0x01) ; 
if ( b )		// An diese PORT n,Bit b erwarte ich einen DS18S20
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x02) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x04) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x08) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x10) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x20) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x40) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}
	
b = (TSONEWI_MASKE&0x80) ; 
if ( b )
	{ 
	pCTSOneWireMehr->bit = b ;
	++pCTSOneWireMehr ;
	}



pCTSOneWireMehr = vCTSOneWireMehr ;
for( i = 0; i < TSONEWI_BITZAHL; ++i, ++pCTSOneWireMehr )
	{
	pCTSOneWireMehr->Endfehlernummer = 1 ;  // PSTR("Es war noch nie gemessen");
	}

romkanal = 0 ;
for(;;)
	{
	/* if ( ++romkanal >= 8 )
		{
		romkanal = 0 ;
		}
	*/
	++romkanal ;
	if ( romkanal >= TSONEWI_BITZAHL ) romkanal = 0 ;
	vCTSOneWireMehr[romkanal].HoleROM();

	// Alle Aufnehmer starten
	pCTSOneWireMehr = vCTSOneWireMehr ;
	for( i = 0; i < TSONEWI_BITZAHL; ++i, ++pCTSOneWireMehr )
		{
		// vTSOneWireMehr[i].StarteMessung();
		pCTSOneWireMehr->StarteMessung();
		// Setzt bei Problem Fehlernummer UND EndFehlernummer
		}

	// Warte Ende der Umwandlung
	tsdelay( sectotick(1.24) );
	pCTSOneWireMehr = vCTSOneWireMehr ;
	for( i = 0; i < TSONEWI_BITZAHL; ++i , ++pCTSOneWireMehr )
		{
		if ( pCTSOneWireMehr->Fehlernummer == 0 )
			{
			// vTSOneWireMehr[i].HoleMesswert();
			pCTSOneWireMehr->HoleMesswert();			
			// Setzt bei Problem Fehlernummer UND EndFehlernummer
			}
		}
	}
}






