// $Id$
/** *****************************************************************************************
@file onewire.cpp

@brief Diese Programm diente zum vorabexperimentieren. 
<br>
Dieses Programm liest einen Temperaturwert aus einem 18DS20 durch dessen One-Wire-Interface aus.


<br>

 Siehe auch <a href=../../../deutsch/ts/onewire.htm>../../../deutsch/ts/onewire.htm</a>
 *************************************************************************/



/*
<br>ROM Kommandos sind:
<br>F0 - Suche ROM : Muss fuer jeden Slave wiederholt werden.
<br>33 - Lese ROM : Den ROM-Inhalt des einzigen Slave auslesen: 8 Bytes einlesen
<br>55 - ROM Vergleichen : 8 Bytes folgen
<br>CC - Alle Slaves ansprechen
<br>EC - grenzwertueberschreitungen suchen.
*/
#define VERSION "2014-06-28 17:23"

#include <heport.h>
#include <tshar.h>
#include <tsser.h>
#include <tsinout.h>
#include <hegettim.h>


#if defined( __TURBOC__ ) || defined ( _MSC_VER )
// Unter DOS / Win95 das printerport verwenden
// #define LPT		// LPT-Port verwenden
#define COM
#endif

#ifdef AVR
#define OMASK 0x40		// 0x40 = PD6  
#define SETZE0 PORTD &= ~OMASK ;	/* 0V ausgeben */ \
		DDRD |= OMASK ;			/* Aktiv */

#define SETZE1 PORTD |= OMASK ;		/* 5V ausgeben */ \
		DDRD |= OMASK ;				/* Aktiv */

#define PASSIV DDRD = ~OMASK ;		/* Nicht Aktiv */ \
		PORTD &= ~OMASK ;		/* aber mit Pull-Up */

#define EINGANG ((PIND & OMASK)!=0)

#else // nicht AVR

// DOS; WIN95
#ifdef COM        // sercon2m
u8 comport4 ;

// Ausgabe an DTR = MOSI
// Einlesen von CTS = MISO

#define OMASK 0x01 // 0x01=DTR , 0x02=RTS
#define SETZE0 comport4 |=  OMASK ;	/* 0V hinter sercon2m */ \
		outportb( 0x3FC, comport4 );
#define SETZE1 lpt &= ~OMASK ;	/* 5V hinter sercon2m */ \
		outportb( 0x3FC, comport4 );
#define PASSIV lpt &= ~OMASK ;	/* 5V */ \
		outportb( 0x3FC, comport4 );
#define EINGANG ( (inportb( 0x3FE ) & 0x10 ) == 0 )
#else
// Linux, Windows
#define SETZE0
#define SETZE1
#define PASSIV
#define EINGANG 0
#endif	// LPT

// DOS; WIN95
#ifdef LPT        // /h/home/html/hjh/etronik/s/pcpartwi.htm
u8 lpt ;
// SCL: OMASK 0x01 - EINGANG = inportb( 0x379 ) & 0x10 )
// SDA: OMASK 0x02 - EINGANG = inprotb( 0x379 ) & 0x20 invertiert
#define OMASK 0x02
#define SETZE0 lpt |=  OMASK ;	/* 0V ausgeben: Transistor SCL ansteuern */ \
		outportb( 0x378, lpt );
#define SETZE1 lpt &= ~OMASK ;	/* 5V ausgeben: Transistor SCL nicht ansteuern */ \
		outportb( 0x378, lpt );
#define PASSIV lpt &= ~OMASK ;	/* 5V ausgeben, der Ausgang ist so aehnlich wie ein offener Kollektor */ \
		outportb( 0x378, lpt );
#define EINGANG ( (inportb( 0x379 ) & 0x20 ) == 0 )
#else
// Linux, Windows
#define SETZE0
#define SETZE1
#define PASSIV
#define EINGANG 0
#endif	// LPT

#endif // nicht AVR

PGM_P fehler ;		// Fehler - String. Zeigt auf Programmspeicher

u32 reset_dt1 ;
u32 reset_dt2 ;
u32 dtick0 ;

u8 bitsgelesen ;	// Anzahl der gelesenen Bits von Reset() bis fehler != 0 
u8 bu[9] ;

#ifdef AVR
#define NSTKDIA 80 // DIA: Stapel frei =43 verwendet =37 von =80
#else
#define NSTKDIA 2000
#endif

void *stkDIA[NSTKDIA] ;		// fun_dialog

// /home/hjh/h/home/html/etronik/sensor/DS18S20.pdf
/*!
 * Den Sensor / Die Kommunikation zuruecksetzen.
 * Falls ein Fehler auftritt, zeigt nachher die globale Variable [fehler] auf den Fehlertext.
 */
FUNCT void Reset(void)
{
u32 starttick ;

bitsgelesen = 0 ;
fehler = 0 ;
SETZE1 ;
delaytick( sectotick( 0.002000 ) );	// 

SETZE0 ;
delaytick( sectotick( 0.000550 ) );	// min 480 us: Macht RESET an allen angeschlossenen Geraeten

PASSIV ;
// Nach dem Reset - Impuls ist die Leitung jetzt wieder High
// Warte auf Low - Der DS1820 sollte jetzt die Leitung nach 0 ziehen.
starttick = gettick();
reset_dt1 = 0 ;
reset_dt2 = 0 ;
for(;;)
	{
	if ( !EINGANG ) break ;	// Low: DS1820 hat die Leitungg LOW gezogen
	// sollte 0.000480 sein
	if ( ( reset_dt1 = difftick( starttick, gettick() ) ) >= sectotick( 0.000480 ) )
		{
		fehler = PSTR("Reset(): wird nicht low") ;
		return ;
		}
	}
// DS1820 hat 0 geschickt
// Jetzt wieder auf 1 warten
for(;;)
	{
	if ( EINGANG ) break ;
	if ( ( reset_dt2 = difftick( starttick, gettick() ) ) >= sectotick( 0.000880 ) )
		{
		fehler = PSTR("Reset(): Leitung am Ende bleibt low") ;
		return ;
		}
	}
// DS1820 hat wieder 1 geschickt
//	reset_dt1=59 Ticks 0.000029 s
//  reset_dt2=289 Ticks 0.000144 s
// INIT-Impuls wurde beantwortet
delaytick( sectotick( 0.000008 ) );
// 0.000008 : CC 64 bringt FE, sonst nichts
// 0.000520 : CC 64 bringt FE, sonst nichts
return ;
}


/*!
 * Schreibe ein 0-Bit, also 60 ... 120 us Dauer, hier 105 us.
 */
FUNCT void Schreibe0(void) // > 60 + 5 us
{
SETZE0 ;
delaytick( sectotick( 0.000105 ) );		// 60 ... 120
SETZE1 ;
delaytick( sectotick( 0.000020 ) );		// 1 ... unbegrenzt
}


/*!
 * Schreibe ein 0-Bit, also 1 ... 15 us Dauer, hier 20 us.
 */
FUNCT void Schreibe1(void) // 10 + 55 us
{
SETZE0 ;
delaytick( sectotick( 0.000020 ) );	// 1 .. 15 us
SETZE1 ;
delaytick( sectotick( 0.000105 ) );	// 60-t0 ... unbegrenzt
}



/*!
 * Lese ein Antwortbit:
 * @return <ul>
 * <li>0 : Eine 0 wurde stabil gelesen.</li>
 * <li>1 : Eine 1 wurde stabil gelesen.</li>
 * <li>2 : Eine 0 blieb nicht stabil anstehen.</li>
 * <li>3 : Eine 1 blieb nicht stabil anstehen.</li>
 * <li>4 : Die 0 geht nicht mehr weg.</li>
 * </ul>
 */
FUNCT u8 Lese(void)
{
u32 starttick ;

delaytick( sectotick( 0.000008 ) );	// 1...unendlich
SETZE0 ;		// Sendeaufforderung
delaytick( sectotick( 0.000002 ) );	// 1  us

PASSIV ;
delaytick( sectotick( 0.000013 ) );	// 15...60 us

starttick = gettick();

if ( EINGANG )
	{	// Eine 1 steht an
	for(;;)
		{
		if ( !EINGANG ) // Fehler
			{
			fehler = PSTR("Lese(): anstehende 1 nicht stabil");
			return 3 ;
			}
		if ( difftick( starttick, gettick() ) >= sectotick( 0.000080 ) )
			{
			if ( fehler == 0 ) ++bitsgelesen ;
			return 1 ;	// Die 1 stand stabil an
			}
		}
	}
else
	{
	dtick0 = 0 ;
	for(;;)
		{
		if ( EINGANG ) // Fehler
			{	// Gesehen bei CC 64
			fehler = PSTR("Lese(): anstehende 0 nicht stabil");
			return 2 ;
			}
		if ( ( dtick0 = difftick( starttick, gettick() ) ) >= sectotick( 0.000005 ) )
			{
			break ;	// Die 0 stand stabil an
			}
		}
// Und jetzt warten, bis die 0 weggeht
	for(;;)
		{
		if ( EINGANG ) // Die ankommende 0 ist weggegangen
			{
			if ( fehler == 0 ) ++bitsgelesen ;
			return 0 ;
			}
		if ( ( dtick0 = difftick( starttick, gettick() ) ) >= sectotick( 0.000090 ) )
			{
			fehler = PSTR("Lese(): anstehende 0 geht nicht weg");
			return 4 ;	// Die 0 geht nicht mehr weg
			}
		}
		
	}
}


/*!
 * Hole 8 Antwortbits, dadurch dass Lese() 8 mal aufgerufen wird.
 */
FUNCT u8 Hole8Bits()
{
u8 b ;
u8 i ;
b = 0 ;
for ( i = 8+1 ; --i ; )
	{	// i = 8,7,6,5,4,3,2,1
	switch( Lese() )
		{
	case 0 : b >>= 1 ; break ;
	case 1 : b >>= 1 ; b |= 0x80 ; break ;
	default :  ; // Lese() hat fehler gesetzt
		}
	}
return b ;
}


/*!
 * Sende 8 Bits, dadurch dass Schreibe0() bzw. Schreibe1() 8 mal aufgerufen wird.
 */
FUNCT void Sende8Bits(u8 b)
{
u8 i ;

for ( i = 8+1 ; --i ; )
	{	// i = 8,7,6,5,4,3,2,1
	if ( b & 0x01 ) 
		{
		Schreibe1() ;
		}
	else
		{
		Schreibe0() ;
		}
	b >>= 1 ;
	}
// i = 0
return ;
}


/*!
 * Falls die globale variable fehler gesetzt wurde, 
 * dann gibt den Fehlertext aus, auf den die globale Variable fehler zeigt.
 */
FUNCT void Meldefehler(void)
{
if ( fehler )
	{	// falls ein Fehler aufgertreten war
	out_puts_P( PSTR(" FEHLER:") );
	out_puts_P( fehler );
	out_puts_P( PSTR("  Bits gelesen:") );
	out_uint8( bitsgelesen );
	out_puts_P( PSTR("  dtick0:") );
	out_uint32( dtick0 );
	out_ulfxdp( dtick0 ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
// Hinweis: Der Compiler berechnet (u32)((4294967296.0)/(double)sectotick(1.000000))
	out_puts_P( PSTR("s") );
	out_nl() ;
	}
}




/*!
 * Zeige den Programmnamen und die Version an.
 */
FUNCT void out_version(void)
{
out_puts_P( PSTR("  \n onewire ") ) ;
out_puts_P( PSTR(VERSION) ) ;
}



/*!
 * Hole 9 Werte nach bu[0] ... bu[8] und zeige diese an.
 */
FUNCT void holezeige8(void)
{
u8 i ;

for ( i = 0 ; i < 9 ; ++i )
	{
	bu[i] = 0x55;	// Irgends etwas, damit ich sehen kann was nicht geholt wurde
	}
for ( i = 0 ; i < 9 ; ++i )
	{
	bu[i] = Hole8Bits();	// Lese noch eine Zahl
//	if ( fehler ) { break ; }
	}


out_puts_P( PSTR("\nGeholt wurde:")  );
for ( i = 0 ; i < 9 ; ++i )
	{
	out_char(' ');
	out_hex2( bu[i] );
	}
out_puts_P( PSTR("  Bits gelesen:") );
out_uint8( bitsgelesen );
out_puts_P( PSTR("  dtick0:") );
out_uint32( dtick0 );
out_char('=');
out_ulfxdp( dtick0 ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
out_puts_P( PSTR("s") );

}



/*!
 * Der Dialog durch die serielle Schnittstelle.
 */
FUNCT void fun_dialog(void)
{
char c ;
u8 i ;
s16 iWert ;

u8 kom1 ;
u8 kom2 ;
// Ticks:
u32 stt ;		// Startzeit
u32 at ;		// Aktueller Zeit
u32 dt ;		// Differenzzeit
u32 dt1 ;		// Differenzzeit
out_version();
kom1 = 0xCC ;
kom2 = 0xBE ;
for(;;)
	{
	SETZE1 ;
	while( in_check() == false )
		{
		tsnext("nochar");
		}
	c = in_char() ;
	switch ( c )
		{
	case '?' :			// Anzeige der Bedienung
		out_puts_P( PSTR("? - Hilfe anzeigen\n"
		"z - Stackverwendung\n"
		"i - Lese ROM\n"
		"l - Lese von Scratchpad\n"
		"r - Reize variabel und Lese\n"
		"s - Schreibe in Scratchpad\n"
		"p - Zeige PORTD / PIND\n"
		"t - Temperatur holen\n"
		"0 - Zeige Ausgabe-Eingang-Verzoegerung\n"
		"1 - Spiele mit kom2 nach gegebenen kom1, z.B: 0xCC\n"
		"2 - Spiele mit kom1 ohne kom2\n"
		"3 - Durch alle kom1, kom2\n" ) );

#ifndef AVR
		out_puts_P( PSTR("q - Programm beenden\n") ) ;
#endif
		out_version();

		break ;

#ifndef AVR
	case 'q' :			// Beenden
		exit(0);
#endif

	case 'z' :			// Verwendeten Stapelspeicher anzeigen
		out_puts_P( PSTR("z: \nDIA: ") ); ts_stack_usage( stkDIA, NSTKDIA );	
		break ;

	case 's' :			// Schreibe einen Wert in Scratchpad
		out_puts_P( PSTR("\ns - Schreibe in Scratchpad ") );
		in_int16_P( PSTR(" Welchen Wert"), &iWert );
		fehler = 0 ;
		Reset();
		if ( fehler ) { Meldefehler(); break ; } 
/*		Sende8Bits( 0xCC );		// Skip ROM Command
		if ( fehler ) { Meldefehler(); break ; } 
*/
		Sende8Bits( 0x4E );	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		Sende8Bits( iWert & 0xff );	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		Sende8Bits( ( iWert >> 8 )  & 0xff);	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		out_puts_P( PSTR("\nOK: Zahl in Puffer gesendet\n") );
/*		Sende8Bits( 0x48 );	// Copy Scratchpad Command
		if ( fehler ) { Meldefehler(); break ; } 
*/
		break ;

		
	case 'i' :		// Lese ROM
		out_puts_P( PSTR("\ni - Lese ROM") );
		kom1 = 0x33 ;

		fehler = 0 ;
		Reset();
		if ( fehler ) { Meldefehler(); break ; } 
//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
		Sende8Bits( kom1 );	// Skip ROM Command 0xCC
		if ( fehler ) { Meldefehler(); break ; } 

		holezeige8();
		if ( fehler ) { Meldefehler(); } 
		
		break ;

		
		// rcc f0 
	case 'r' :		// Reize und Lese den Wert von Scratchpad
		out_puts_P( PSTR(	"\nr - Reize und Lese"
							"\nCC 33 - Ueberspringe ROM"
							"\nBE 7D - Schmierspeicher holen"
							"\n55 AA - Waehle ROM aus"
							"\n48 12 - Schmierspeicher 2 und 3 sichern nach EEPROM"
							"\nB8 1D - Schmierspeicher 2 und 3 von EEPROM wieder holen"
							"\n4E 72 - Bytes 2 und 3 des Schmierspeichers veraendern"
							"\nB4 2D - Spannungsversorgung abfragen"
							"\n44 22 - Temperatur erfassen"
							"\nF0 0F - Suche Geraet"
							 ) );
		if ( in_hex8_P( PSTR("Vorkommando 1 "), &kom1 ) ) break ;
		if ( in_hex8_P( PSTR("Vorkommando 2 "), &kom2 ) ) break ;
		goto M_l;	
		
		
	case 'l' :		// Lese den Wert von Scratchpad
		out_puts_P( PSTR("\nl - Lese von Scratchpad") );
		kom1 = 0xCC ;
		kom2 = 0xBE ;

M_l:;

		fehler = 0 ;
		Reset();
		if ( fehler ) { Meldefehler(); }
//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
		Sende8Bits( kom1 );	// Skip ROM Command 0xCC
		if ( fehler ) { Meldefehler(); }

		Sende8Bits( kom2 );	// Lese alle 9 Zellen
		if ( fehler ) { Meldefehler(); }
		
		holezeige8();
		if ( fehler ) { Meldefehler(); } 

// Zeiten ausgeben
		out_puts_P( PSTR("  reset_dt1=" ) );
		out_uint32( reset_dt1 );
		out_puts_P( PSTR(" Ticks ") );
		out_ulfxdp( reset_dt1 ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
		out_puts_P( PSTR(" s") );

		out_puts_P( PSTR("  reset_dt2=" ) );
		out_uint32( reset_dt2 );
		out_puts_P( PSTR(" Ticks ") );
		out_ulfxdp( reset_dt2 ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
		out_puts_P( PSTR(" s") );
		break ;

	case 't' : 		// Lese den Temperaturwert
		out_puts_P( PSTR("\nt - Temperatur ") );
		while( in_check() == false )
			{
			fehler = 0 ;
			Reset();
			if ( fehler ) { Meldefehler(); break ; } 
			Sende8Bits( 0xCC );
			if ( fehler ) { Meldefehler(); break ; } 
			Sende8Bits( 0x44 );	// Convert T
			if ( fehler ) { Meldefehler(); break ; } 
	/*		holezeige8();
			if ( fehler ) { Meldefehler(); break ; } 
	*/
			tsdelay( sectotick( 0.755 ) );	 // Warte 750 ms
			Reset();
			Sende8Bits( 0xCC );
			if ( fehler ) { Meldefehler(); break ; } 
			Sende8Bits( 0xBE );	// Lese alle 9 Zellen
			if ( fehler ) { Meldefehler(); break ; } 
			holezeige8();
			if ( fehler ) { Meldefehler(); break ; } 


			out_puts_P( PSTR("\nZeit:")  );
			out_ulfxdp( gettick() ,6,(u32)((4294967296.0)/(double)sectotick(1.0)), 0 ); 
			out_puts_P( PSTR("s  Temperatur:")  );
			out_swfxdp( *(s16 *)(&bu[0]) ,1,16384, 0 );

			out_puts_P( PSTR("grdC")  );
			out_nl();
			}
		break ;

	case 'p' :
#ifdef AVR
		out_puts_P( PSTR("\np - PIND=") );
		out_hex2( PIND );
		out_puts_P( PSTR(" PORTD=") );
		out_hex2( PORTD );
		out_puts_P( PSTR(" DDRD=") );
		out_hex2( DDRD );
#endif
#ifdef LPT
		out_puts_P( PSTR("\np - LPT 0x378=") );
		out_hex2( inportb(0x378) );
		out_puts_P( PSTR("  lpt=") );
		out_hex2( lpt );
		out_puts_P( PSTR(" DDRD=") );
		out_hex2( inportb(0x379) );
#endif
		break ;



	case '0' :

M0Menu:
		out_puts_P( PSTR( "\n ... Taste q=Abbruch, 0=Null, 1=Eins-" ) );
		c = in_char() ;
		if ( c == 'q' ) goto M0q ;
		if ( c == '0' ) goto M00 ;
		if ( c == '1' ) goto M01 ;

		// Pruefprogramm 0
M01:	out_puts_P( PSTR("\n0 - Setze 1") );
		fehler = 0 ;
		stt = gettick();
		SETZE1 ;
		for(;;)
			{		// Warte auf Ruekmeldung
			at = gettick();
			dt = difftick( stt,at);

			if ( EINGANG )
				{			// die 1 ist zurueckgekommen.
				break ;
				}
			if ( dt > sectotick( 0.1 ) )
				{			// Timeout
				out_puts_P( PSTR("0.0: TIMEOUT: trotz Ausgabe 1 kommt Rueckmeldung 0") ) ;
				break ;
				}
			}

		out_puts_P( PSTR("\n0:0: x->1 nach " ) ) ;
		out_uint32( dt );
		out_puts_P( PSTR(" Ticks ") );
		out_ulfxdp( dt ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
		out_puts_P( PSTR(" s") );
		goto M0Menu ;


M00:	out_puts_P( PSTR("\n0 - Setze 0") );
		stt = gettick();
		SETZE0 ;
		for(;;)
			{
			at = gettick();
			dt1 = difftick( stt,at);

			if ( !EINGANG ) break ;
			if ( dt1 > sectotick( 0.10000 ) )
				{
				out_puts_P( PSTR("0.1: TIMEOUT: trotz Ausgabe 0 kommt Rueckmeldung 1") ) ;
				break ;
				}
			}
		out_puts_P( PSTR("\n0.1: 1->0 nach " ) ) ;
		out_uint32( dt1 );
		out_puts_P( PSTR(" Ticks ") );
		out_ulfxdp( dt1 ,6,(u32)((4294967296.0)/(double)sectotick(1.000000)), 0 );
		out_puts_P( PSTR(" s") );
		goto M0Menu ;


M0q:
		break ;


	case '1':
		// CC 64 = FE FF FF FF  FF FF FF FF
		out_puts_P( PSTR("\n1 : Spielel mit 2. Kommando nach kom1\n") );
		kom1 = 0xCC ;
		if ( in_hex8_P( PSTR("kom1 "), &kom1 ) ) break ;
		for ( kom2 = 0 ; ++kom2 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" kom2=") );
			out_hex2( kom2 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			
			Reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			Sende8Bits( kom1 );	// Skip ROM Command 0xCC
			if ( fehler ) { Meldefehler(); break; ; } 

			Sende8Bits( kom2 );	// ??
			if ( fehler ) { Meldefehler(); continue ; } 
		
			holezeige8();
			if ( fehler ) { Meldefehler();  } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2
		break ;


	case '2':

		out_puts_P( PSTR("\n2 : Spiele mit einem Kommando\n") );
		for ( kom1 = 0 ; ++kom1 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			Reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			Sende8Bits( kom1 );
			if ( fehler ) { Meldefehler(); break; ; } 

		
			holezeige8();
			if ( fehler ) { Meldefehler(); } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2
		break ;


	case '3':

		for ( kom1 = 0 ; ++kom1 ; )
		for ( kom2 = 0 ; ++kom2 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" kom2=") );
			out_hex2( kom2 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			Reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			Sende8Bits( kom1 );	// Skip ROM Command 0xCC
			if ( fehler ) { Meldefehler(); break; ; } 

			Sende8Bits( kom2 );	// ??
			if ( fehler ) { Meldefehler(); continue ; } 
		
			holezeige8();
			if ( fehler ) { Meldefehler();  } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2 / kom1
		break ;


	case 0x0d:
	case 0x0a:
	case 0x09:
	case 0x20:
		break ;

	default:
		out_puts_P( PSTR("\n* Falsches Kommando. Hilfe=?\n") );
		break ;
		}
	out_puts_P( PSTR("\n") );
	tsnext("fun_dialog");
	}
}



/*!
 * Das Hauptprogramm. Initialisieren und das Zeitverwaltungssystem tshar starten.
 */
FUNCT int main( int argc, const char **argv )
{
tsseropen();

ts_stack_fill( stkDIA, NSTKDIA );	// Muster im STACK ablegen fuer Messung der Belegung

tickopen();
tsopen();		// Tabellen initialisieren, 


/*
vout_char[1] = tsser_out_char ; 
vin_char[1] = tsser_in_char ;
vin_check[1] = tsser_in_check ;

	vout_char[i] = tsser_out_char ; 
	vin_char[i] = tsser_in_char ;
	vin_check[i] = tsser_in_check ;

*/

tseinri( 1 , fun_dialog, "fun_dialog", stkDIA, sizeof(stkDIA) );

tsnext( "main" );	// run one piece of fun_dialog

for(;;)
	{
	tsnext("main");
	}

return 0 ;		// Wird nie zurueckkehren
}


