/************************************************************************/
/*																		*/
/*			Access Dallas 1-Wire Devices								*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
// /home/hjh/www/net/mikrocontroller/topic/14792/1wire_Mega16
// #include "main.h"

#define VERSION "2014-06-22"


#include <heport.h>

#include <tshar.h>
#include <tsser.h>
#include <tsinout.h>
#include <hegettim.h>




#ifndef _1wire_h_
#define _1wire_h_
#define MATCH_ROM	0x55
#define SKIP_ROM	0xCC
#define	SEARCH_ROM	0xF0

#define CONVERT_T	0x44		// DS1820 commands
#define READ		0xBE
#define WRITE		0x4E
#define EE_WRITE	0x48
#define EE_RECALL	0xB8

#define	SEARCH_FIRST	0xFF		// start new search
#define	PRESENCE_ERR	0xFF
#define	DATA_ERR	0xFE
#define LAST_DEVICE	0x00		// last device found
//			0x01 ... 0x40: continue searching

void w1_reset(void);

u8 w1_byte_wr( u8 b );
u8 w1_byte_rd( void );

u8 w1_rom_search( u8 diff, u8 *id );

void w1_command( u8 command, u8 *id );
#endif




#ifndef W1_PIN
#define W1_PIN	PD6
#define W1_IN	PIND
#define W1_OUT	PORTD
#define W1_DDR	DDRD
#define SETZE1 W1_DDR |= 1 << PD6 ; W1_OUT |= 1 << PD6 ;
#define SETZE0 W1_DDR |= 1 << PD6 ; W1_OUT &= ~(1 << PD6) ;
#define PASSIV W1_DDR = ~(1 << PD6) ; W1_OUT |= (1 << PD6) ;
#define EINGANG ((W1_IN & (1 << PD6))!=0)
#endif

PGM_P fehler ;		// Fehler - String. Zeigt auf Programmspeicher

u8 bu[9] ;

u32 reset_dt1 ;
u32 reset_dt2 ;

#ifdef AVR
#define NSTKDIA 80 // Stack1 unused = of =
#else
#define NSTKDIA 2000
#endif

void *stkDIA[NSTKDIA] ;		// fun_dialog



/*!
 * Fehlerkennung : fehler wird gesetzt (!= 0)
 */
void w1_reset(void)
{
u32 starttick ;

SETZE1 ;
delaytick( sectotick( 0.002000 ) );	// 

SETZE0 ;
delaytick( sectotick( 0.000500 ) );	// min 480 us: Macht RESET an allen angeschlossenen Geraeten

PASSIV ;
/*
starttick = gettick();
for(;;)
	{
	if ( EINGANG ) break ;
	if ( difftick( starttick, gettick() ) >= sectotick( 0.000015 ) )
		{
		fehler = PSTR("Reset(): Leitung bleibt low") ;
		return ;
		}
	}
// Nach dem Reset - Impuls ist die Leitung jetzt wieder High
// Warte auf Low - Der DS1820 sollte jetzt die Leitung nach 0 ziehen.
*/
starttick = gettick();
for(;;)
	{
	if ( !EINGANG ) break ;	// Low
	// sollte 0.000480 sein
	if ( ( reset_dt1 = difftick( starttick, gettick() ) ) >= sectotick( 0.000480 ) )
		{
		fehler = PSTR("Reset(): Leitung wird nicht Low") ;
		return ;
		}
	}
// 18DS20 hat 0 geschickt
for(;;)
	{
	if ( EINGANG ) break ;
	if ( ( reset_dt2 = difftick( starttick, gettick() ) ) >= sectotick( 0.000480 ) )
		{
		fehler = PSTR("Reset(): Leitung am Ende bleibt low") ;
		return ;
		}
	}
// 18DS20 hat wieder 1 geschickt
//	reset_dt1=59 Ticks 0.000029 s
//  reset_dt2=289 Ticks 0.000144 s
// INIT-Impuls wurde beantwortet
for(;;)
	{
	if ( difftick( starttick, gettick() ) >= sectotick( 0.000960 ) )
		{
		return ;
		}
	}
return ;
}

/*{

  W1_OUT &= ~(1<<W1_PIN);
  W1_DDR |= 1<<W1_PIN;
  delaytick( sectotick( 0.000480 ));			// 480 us
//  cli();
  W1_DDR &= ~(1<<W1_PIN);
  delaytick( sectotick( 0.000066 ));	

  if ( W1_IN & (1<<W1_PIN) )  fehler = PSTR("Error nichts angeschlossen");
//  sei();
  delaytick( sectotick( 0.000480 - 0.000066 ));
  if( (W1_IN & (1<<W1_PIN)) == 0 )		// short circuit
    fehler = PSTR("Error Kurzschluss");
  return ;
} */


u8 w1_bool_io( bool b )
{
//  cli();
  W1_DDR |= 1<<W1_PIN;
  delaytick( sectotick( 0.000001 ));
  if( b )
    W1_DDR &= ~(1<<W1_PIN);
  delaytick( sectotick( 0.000015 - 0.000001 ));
  if( (W1_IN & (1<<W1_PIN)) == 0 )
    b = 0;
  delaytick( sectotick( 0.000060 - 0.000015 ));
  W1_DDR &= ~(1<<W1_PIN);
//  sei();
  return b;
}


u8 w1_byte_wr( u8 b )
{
  u8 i = 8, j;
  do{
    j = w1_bool_io( b & 1 );
    b >>= 1;
    if( j )
      b |= 0x80;
  }while( --i );
  return b;
}


u8 w1_byte_rd( void )
{
  return w1_byte_wr( 0xFF );
}


u8 w1_rom_search( u8 diff, u8 *id )
{
  u8 i, j, next_diff;
  bool b;

  fehler = 0 ;
  
  if( fehler )
    return PRESENCE_ERR;			// error, no device found
  w1_byte_wr( SEARCH_ROM );			// ROM search command
  next_diff = LAST_DEVICE;			// unchanged on last device
  i = 8 * 8;					// 8 bytes
  do{
    j = 8;					// 8 bools
    do{
      b = w1_bool_io( 1 );			// read bool
      if( w1_bool_io( 1 ) ){			// read complement bool
	if( b )					// 11
	  return DATA_ERR;			// data error
      }else{
	if( !b ){				// 00 = 2 devices
	  if( diff > i ||
	    ((*id & 1) && diff != i) ){
	    b = 1;				// now 1
	    next_diff = i;			// next pass 0
	  }
	}
      }
      w1_bool_io( b );     			// write bool
      *id >>= 1;
      if( b )					// store bool
	*id |= 0x80;
      i--;
    }while( --j );
    id++;					// next byte
  }while( i );
  return next_diff;				// to continue search
}


void w1_command( u8 command, u8 *id )
{
  u8 i;
  w1_reset();
  if( id ){
    w1_byte_wr( MATCH_ROM );			// to a single device
    i = 8;
    do{
      w1_byte_wr( *id );
      id++;
    }while( --i );
  }else{
    w1_byte_wr( SKIP_ROM );			// to all devices
  }
  w1_byte_wr( command );
}







FUNCT void Meldefehler(void)
{
if ( fehler )
	{
	out_puts_P( PSTR(" FEHLER:") );
	out_puts_P( fehler ) ;
	out_nl() ;
	}
}




FUNCT void out_version(void)
{
out_puts_P( PSTR("  \n onewire ") ) ;
out_puts_P( PSTR(VERSION) ) ;
}




FUNCT void holezeige8(void)
{
u8 i ;

for ( i = 0 ; i < 9 ; ++i )
	{
	bu[i] = 0x55;	// Irgends etwas
	}
for ( i = 0 ; i < 9 ; ++i )
	{
	bu[i] = w1_byte_rd();	// Lese noch eine Zahl
	if ( fehler ) { return ; } 
	}
/* if ( bu[0] != 0xff || 
	 bu[1] != 0xff || 
	 bu[2] != 0xff || 
	 bu[3] != 0xff || 
	 bu[4] != 0xff || 
	 bu[5] != 0xff || 
	 bu[6] != 0xff || 
	 bu[7] != 0xff || 
	 bu[8] != 0xff ) */
	{
	out_puts_P( PSTR("\nOK: Zahl in Puffer ist:")  );
	for ( i = 0 ; i < 9 ; ++i )
		{
		out_char(' ');
		out_uint8(i);
		out_char('=');
		out_hex2( bu[i] );
		}
	if ( fehler ) { return ; } 
	}
}


// Der Dialog durch die serielle Schnittstelle
FUNCT void fun_dialog(void)
{
char c ;
u8 i ;
s16 iWert ;

u8 kom1 ;
u8 kom2 ;
// Ticks:
/*
u32 stt ;		// Startzeit
u32 at ;		// Aktueller Zeit
u32 dt ;		// Differenzzeit
u32 dt1 ;		// Differenzzeit
*/
out_version();
kom1 = 0xCC ;
kom2 = 0xBE ;
for(;;)
	{
//	SETZE1 ;
	while( in_check() == false )
		{
		tsnext("nochar");
		}
	c = in_char() ;
	switch ( c )
		{
	case '?' :			// Anzeige der Bedienung
		out_puts_P( PSTR("? - Hilfe anzeigen\n"
		"z - Stackverwendung\n"
		"i - Lese ROM\n"
		"l - Lese von Scratchpad\n"
		"r - Reize variabel und Lese\n"
		"s - Schreibe in Scratchpad\n"
		"p - Zeige PORTD / PIND\n"
		"t - Temperatur holen\n"
		"1 - Spiele mit kom2 nach 0xCC\n"
		"2 - Spiele mit kom1\n"
		"3 - Spiele mit kom1 und Kom2\n" ) );

#ifndef AVR
		out_puts_P( PSTR("q - Programm beenden\n") ) ;
#endif
		out_version();

		break ;

#ifndef AVR
	case 'q' :			// Beenden
		exit(0);
#endif

	case 'z' :			// Verwendeten Stapelspeicher anzeigen
		out_puts_P( PSTR("z: \nDIA: ") ); ts_stack_usage( stkDIA, NSTKDIA );	
		break ;

	case 's' :			// Schreibe einen Wert in Scratchpad
		out_puts_P( PSTR("\ns - Schreibe in Scratchpad ") );
		in_int16_P( PSTR(" Welchen Wert"), &iWert );
		fehler = 0 ;
		w1_reset();
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( 0xCC );		// Skip ROM Command
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( 0x4E );	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( iWert & 0xff );	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( ( iWert >> 8 )  & 0xff);	// Schreibe in die 2 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		out_puts_P( PSTR("\nOK: Zahl in Puffer gesendet\n") );
/*		w1_byte_wr( 0x48 );	// Copy Scratchpad Command
		if ( fehler ) { Meldefehler(); break ; } 
*/
		break ;

		
	case 'i' :		// Lese ROM
		out_puts_P( PSTR("\ni - Lese ROM") );
		kom1 = 0x33 ;

		fehler = 0 ;
		w1_reset();
		if ( fehler ) { Meldefehler(); break ; } 
//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
		w1_byte_wr( kom1 );	// Skip ROM Command 0xCC
		if ( fehler ) { Meldefehler(); break ; } 

		holezeige8();
		if ( fehler ) { Meldefehler(); } 
		
		break ;

		
		// rcc f0 
	case 'r' :		// Reize und Lese den Wert von Scratchpad
		out_puts_P( PSTR(	"\nr - Reize und Lese"
							"\nCC 33 - Ueberspringe ROM"
							"\nBE 7D - Schmierspeicher holen"
							"\n55 AA - Waehle ROM aus"
							"\n48 12 - Schmierspeicher 2 und 3 sichern nach EEPROM"
							"\nB8 1D - Schmierspeicher 2 und 3 von EEPROM wieder holen"
							"\n4E 72 - Bytes 2 und 3 des Schmierspeichers veraendern"
							"\nB4 2D - Spannungsversorgung abfragen"
							"\n44 22 - Temperatur erfassen"
							"\nF0 0F - Suche Geraet"
							 ) );
		if ( in_hex8_P( PSTR("Vorkommando 1 "), &kom1 ) ) break ;
		if ( in_hex8_P( PSTR("Vorkommando 2 "), &kom2 ) ) break ;
		goto M_l;	
		
		
	case 'l' :		// Lese den Wert von Scratchpad
		out_puts_P( PSTR("\nl - Lese von Scratchpad") );
		w1_reset();
		if ( fehler ) { Meldefehler(); break ; }	
		w1_command( 0xBE, 0 );
		if ( fehler ) { Meldefehler(); break ; }	
		holezeige8();
		if ( fehler ) { Meldefehler(); break ; }	
		break ;
		
M_l:;

		fehler = 0 ;
		w1_reset();
		if ( fehler ) { Meldefehler(); break ; } 
//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
		w1_byte_wr( kom1 );	// Skip ROM Command 0xCC
		if ( fehler ) { Meldefehler(); break ; } 

		w1_byte_wr( kom2 );	// Lese alle 9 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		
		holezeige8();
		if ( fehler ) { Meldefehler(); } 

// Zeiten ausgeben
		break ;

	case 't' : 		// Lese den Temperaturwert
		out_puts_P( PSTR("\nt - Temperatur ") );
		fehler = 0 ;
		w1_reset();
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( 0xCC );
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( 0x44 );	// Convert T
		if ( fehler ) { Meldefehler(); break ; } 
		tsdelay( sectotick( 0.010 ) );
		w1_byte_wr( 0xCC );
		if ( fehler ) { Meldefehler(); break ; } 
		w1_byte_wr( 0xBE );	// Lese alle 9 Zellen
		if ( fehler ) { Meldefehler(); break ; } 
		for ( i = 0 ; i < 9 ; ++i )
			{
			bu[i] = w1_byte_rd();	// Lese noch eine Zahl
			if ( fehler ) { Meldefehler(); break ; } 
			}
		if ( fehler ) { Meldefehler(); break ; } 
		out_puts_P( PSTR("\nOK: Zahl in Puffer ist:")  );
		for ( i = 0 ; i < 9 ; ++i )
			{
			out_char(' ');
			out_hex2( bu[i] );
			}

		break ;

	case 'p' :
		out_puts_P( PSTR("\np - PIND=") );
		out_hex2( PIND );
		out_puts_P( PSTR(" PORTD=") );
		out_hex2( PORTD );
		out_puts_P( PSTR(" DDRD=") );
		out_hex2( DDRD );
		break ;

	case '1':
		// CC 64 = FE FF FF FF  FF FF FF FF
		kom1 = 0xCC ;
		for ( kom2 = 0 ; ++kom2 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" kom2=") );
			out_hex2( kom2 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			w1_reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			w1_byte_wr( kom1 );	// Skip ROM Command 0xCC
			if ( fehler ) { Meldefehler(); break; ; } 

			w1_byte_wr( kom2 );	// ??
			if ( fehler ) { Meldefehler(); continue ; } 
		
			holezeige8();
			if ( fehler ) { Meldefehler();  } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2
		break ;


	case '2':

		for ( kom1 = 0 ; ++kom1 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			w1_reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			w1_byte_wr( kom1 );
			if ( fehler ) { Meldefehler(); break; ; } 

		
			holezeige8();
			if ( fehler ) { Meldefehler(); } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2
		break ;


	case '3':

		for ( kom1 = 0 ; ++kom1 ; )
		for ( kom2 = 0 ; ++kom2 ; )
			{
			out_puts_P( PSTR("\nkom1=") );
			out_hex2( kom1 );
			out_puts_P( PSTR(" kom2=") );
			out_hex2( kom2 );
			out_puts_P( PSTR(" ") );
			fehler = 0 ;
			w1_reset();
			if ( fehler ) { Meldefehler(); break ; } 
	//		out_puts_P( PSTR("\nReset() erfolgreich") );
		
			w1_byte_wr( kom1 );	// Skip ROM Command 0xCC
			if ( fehler ) { Meldefehler(); break; ; } 

			w1_byte_wr( kom2 );	// ??
			if ( fehler ) { Meldefehler(); continue ; } 
		
			holezeige8();
			if ( fehler ) { Meldefehler();  } 
			if ( bu[0] != 0xff || 
				 bu[1] != 0xff || 
				 bu[2] != 0xff || 
				 bu[3] != 0xff || 
				 bu[4] != 0xff || 
				 bu[5] != 0xff || 
				 bu[6] != 0xff || 
				 bu[7] != 0xff ) 
				{
				c = in_char() ;
				if ( c == 'q' ) break ;
				}
			}		// next kom2 / kom1
		break ;


	case 0x0d:
	case 0x0a:
	case 0x09:
	case 0x20:
		break ;

	default:
		out_puts_P( PSTR("\n* Falsches Kommando. Hilfe=?\n") );
		break ;
		}
	out_puts_P( PSTR("\n") );
	tsnext("fun_dialog");
	}
}




// The main program
FUNCT int main( int argc, const char **argv )
{
tsseropen();

ts_stack_fill( stkDIA, NSTKDIA );	// Muster im STACK ablegen fuer Messung der Belegung

tickopen();
tsopen();		// Tabellen initialisieren, 


/*
vout_char[1] = tsser_out_char ; 
vin_char[1] = tsser_in_char ;
vin_check[1] = tsser_in_check ;

	vout_char[i] = tsser_out_char ; 
	vin_char[i] = tsser_in_char ;
	vin_check[i] = tsser_in_check ;

*/

tseinri( 1 , fun_dialog, "fun_dialog", stkDIA, sizeof(stkDIA) );

tsnext( "main" );	// run one piece of fun_dialog

for(;;)
	{
	tsnext("main");
	}

return 0 ;		// return - will never return !
}



