// $Id$
/** *****************************************************************************************
@doxygen

@file tsonew_n.cpp

@brief Demo der Abfragen von Dallas DS1820 One-Wire-Thermometer. 
Hierdurch werden die Unterprogramme demonstriert, die 8 Dallas DS18S20 Temperatursensoren abfragen.
<br>
Siehe auch <a href=../../../deutsch/ts/tsonew_n.htm>../../../deutsch/ts/tsonew_n.htm</a> - Anwendungsbeschreibung
<br>
<a href=../../../../../etronik/sensor/DS18S20.pdf>../../../../../etronik/sensor/DS18S20.pdf</a> - Der Sensor
<br>
<a href=../../lq/html/tsonewin_8cpp.html>../../lq/html/tsonewin_8cpp.html</a> - Die Unterprogramme

@author HJHerbert

@version 2014-08-16 - 
@version 2014-08-19 - Test und einige kleinere Verbesserungen.
@version 2015-09-27 - Umstellung auf mehrere Sensoren, einen pro Port-Bit
@version 2015-10-11 - 
@version 2015-10-13 - Anzeigeauflsung erhoeht, nicht nur 0.0 / 0.5 - Schritte
@version 2015-11-06 - Verzgerungen geaendert
@version 2016-11-28 - An das neue lq/tsonewin.cpp angepasst


********************************************************************************************* */


#define VERSION "2016-11-28"

#define DB_ENABLED 0
#define DB_INFO 0


#include <heport.h>
#include <tshar.h>
#include <tsser.h>
#include <tsinout.h>
#include <hegettim.h>
#include <tsonewin.h>

#include <hedeb.h>


// u8 bu[SERNUM] ;





#ifdef AVR
#define NSTKDIA 64
#else
#define NSTKDIA 2000
#endif

void *stkDIA[NSTKDIA] ;		// fun_dialog


#ifdef AVR
#define NSTKONEWIN 76
#else
#define NSTKONEWIN 2000	// MDOS50 benoetigt mehr als 512
#endif


void *stkONEWIN[NSTKONEWIN];


u8 iKanal ;	// Init-Wert = 0 
u8 bitKanal = 1 ;

/*!
 * Version (Datum) anzeigen
 */
FUNCT void out_version(void)
{
out_puts_P( PSTR( " tsonew_n Version"
                  VERSION
				  "\n" ) ) ;
}


/*!
 *
 * @param n - Fehlernummer / Endfehlerenummer aus lq/tsonewin.cpp
 */
FUNCT void Fehlerausgabe(u8 n) 
{

if ( n  == 0 )
	{
	out_puts_P( PSTR(" OK" ));
	return ;
	}
	
out_puts_P( PSTR("Fehlernummer: ")) ;
out_uint8( n );
switch( n / 10 ) {	// In welchem Aufruf
	case 0:  break ;
	case 1:  out_puts_P( PSTR(" Beim 1. Reset: "		));	break ;
	case 2:  out_puts_P( PSTR(" Beim 1. Skip Rom: "		));	break ;
	case 3:  out_puts_P( PSTR(" Bei Wandlung start : "	));	break ;
	case 4:  out_puts_P( PSTR(" Beim 2. Reset: "		));	break ;
	case 5:  out_puts_P( PSTR(" Beim 2. Skip Rom: "		));	break ;
	case 6:  out_puts_P( PSTR(" Bei Leseanforderung: "	));	break ;
	case 7:  out_puts_P( PSTR(" Beim Lesen der Werte: "	));	break ;
	default: out_puts_P( PSTR(" Unbekannt"				));	break ;
	}

switch( n % 10 ) {	// Welcher Teilfehler
	case 0 : break ;
	case 1: out_puts_P( PSTR("Es war noch nie gemessen")) ;		break ;
	case 2: out_puts_P( PSTR("Reset(): Leitung wird nicht Low")) ;		break ;
	case 3: out_puts_P( PSTR("Reset(): Leitung am Ende bleibt Low")) ;	break ;
	case 4: out_puts_P( PSTR("Lese(): anstehende 1 nicht stabil"));		break ;
	case 5: out_puts_P( PSTR("Lese(): anstehende 0 nicht stabil"));		break ;
	case 6: out_puts_P( PSTR("Lese(): anstehende 0 geht nicht weg"));	break ;
	case 7: out_puts_P( PSTR("Hole8Bytes(): nur ff gekommen"));			break ;
	default: out_puts_P( PSTR("Unbekannt"));			break ;
	}
}


FUNCT void oTemperatur(u8 i)
{
out_char( ',' );
out_char( 'k' );
out_uint8(i);
out_char( ',' );
if ( i < TSONEWI_BITZAHL )
	{
	if ( vCTSOneWireMehr[i].Endfehlernummer == 0 )
		{
		out_swfxdp( vCTSOneWireMehr[i].Temperaturwert,				// p1 Wert s16
		2,							// p2 Anzeige-Nachkommastellen
		3277,						// p3 Normierung: Wert in 1/100 grd C grdC, also 32768/100=327.68
		1 );						// p4 eine Nachkommastelle in p3 Normierung
									// p3/10^p4=327.7
		}
	else
		{
		out_puts_P( PSTR("Fehler#"));
		out_uint8(vCTSOneWireMehr[i].Endfehlernummer);
		Fehlerausgabe(vCTSOneWireMehr[i].Fehlernummer);
		}
	}
else
	{
	out_puts_P( PSTR("FehlerKanalNr"));
	}
out_char( ',' );
}


/*! 
 * Der Dialog durch die serielle Schnittstelle.
 */
FUNCT void fun_dialog(void)
{
char c ;
int i ;
int j ;
CTSOneWireMehr* pCTSOneWireMehr;
out_version();

for(;;)
	{
	out_puts_P( PSTR("tsonew_n: "));
	while( in_check() == false )
		{		// Warte, bis ein Zeichen vom Dialog kommt.
		tsnext("nochar");
		}
	c = in_char() ;	// Dialog: Das Befehlszeichen
	out_char(c);
	out_char(' ');
	switch ( c )
		{
	case '?' :			// Anzeige der Bedienung
		out_puts_P( PSTR("\n? - Hilfe anzeigen\n"
		"z - Stackverwendung\n"
		"0.." ));
		out_uint8( TSONEWI_BITZAHL -1 );
		out_puts_P( PSTR(" - eine Temperatur laufend anzeigen\n"
		"t - Temperaturen anzeigen\n"
		"b - Speicher Hex anzeigen\n"
		) );

#ifndef AVR
		out_puts_P( PSTR("q - Programm beenden\n") ) ;
#endif
		out_version();

		break ;

#ifndef AVR
	case 'q' :			// Beenden
		DBexit();
		exit(0);
#endif

	case 'z' :			// Verwendeten Stapelspeicher anzeigen
		out_puts_P( PSTR("\nDIA: ") ); ts_stack_usage( stkDIA, NSTKDIA );
		out_puts_P( PSTR("\nONEWIN: ") ); ts_stack_usage( stkONEWIN, NSTKONEWIN );
		break ;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7': 		// Lese den Temperaturwert i alle 10 Sekunden ein Mal
		if ( c-'0' < TSONEWI_BITZAHL )
			{
			out_puts_P( PSTR("\nDS1820[") );
			out_uint8(c-'0');
			out_puts_P( PSTR("]: ") );
			for(;;)
				{
				if( in_check() )
					{		// Warte, bis ein Zeichen vom Dialog kommt.
					break ;
					}
				tsnext("nochar");
				out_char( ',');
				out_sec(gettick());
				out_char( ',');
				oTemperatur(c-'0');	// Kanalnummer, Wert und Fehler anzeigen
				out_nl();
				tsdelay( sectotick(2.0) );
				}
			}
		else
			{
			out_puts_P( PSTR( "Index nicht erlaubt\n" ) );
			}
		break ;
		
	case 't' : 		// Lese den Temperaturwert
		out_puts_P( PSTR("Temperaturen \n") );
		for(;;)
			{
			if( in_check() )
				{		// Warte, bis ein Zeichen vom Dialog kommt.
				break ;
				}
			out_sec( gettick() );
			for( i = 0; i < TSONEWI_BITZAHL; ++i )
				{
				oTemperatur(i);	// Kanalnummer, Wert und Fehler anzeigen
				}
			out_nl();
			tsdelay( sectotick(2.0) );
			}
		break ;

	case 'b' : 		// Alle Buffer anzeigen
		out_puts_P( PSTR("Buffer") );
		pCTSOneWireMehr = vCTSOneWireMehr ;
		for( i = 0; i < TSONEWI_BITZAHL; ++i , ++pCTSOneWireMehr )
			{
			oTemperatur(i);	// Kanalnummer, Wert und Fehler anzeigen
			out_puts_P( PSTR( "  bu=" ));
			for( j = 0 ; j < SERNUMLEN ; ++j )
				{
				out_char(',');
				out_hex2( pCTSOneWireMehr->bu[j] );
				}
			out_puts_P( PSTR( "  bus=" ));
			for( j = 0 ; j < SERNUMLEN ; ++j )
				{
				out_char(',');
				out_hex2( pCTSOneWireMehr->bus[j] );
				}
			out_nl();
			}
		break ;
	case 0x0d:
	case 0x0a:
	case 0x09:
	case 0x20:
		out_version();
		break ;
	case 0x1b:
		break;
	default:
		out_puts_P( PSTR("\n* Falsches Kommando. Hilfe=?\n") );
		break ;
		}
	out_puts_P( PSTR("\n") );
	tsnext("fun_dialog");
	}
}



/*!
 * Das Hauptprogramm
 * Hier wird alles eingerichtet und gestartet.
 */
FUNCT int main( int argc, const char **argv )
{
DBmain("tsonew_n.dbg");

tsseropen();

ts_stack_fill( stkDIA, NSTKDIA );			// Muster im STACK ablegen fuer Messung der Belegung
ts_stack_fill( stkONEWIN, NSTKONEWIN );	// Muster im STACK ablegen fuer Messung der Belegung
tickopen();
tsopen();		// Tabellen initialisieren, 


/*
vout_char[1] = tsser_out_char ; 
vin_char[1] = tsser_in_char ;
vin_check[1] = tsser_in_check ;

	vout_char[i] = tsser_out_char ; 
	vin_char[i] = tsser_in_char ;
	vin_check[i] = tsser_in_check ;

*/

tseinri( 1 , fun_dialog, "fun_dialog", stkDIA, sizeof(stkDIA) );

// Diese Task befindet sich in lq/tsonewin.cpp, definiert in include/tsonewin.h
tseinri( 2 , CTSOneWireMehr__fun_one_wire, "fun_one_wire", stkONEWIN, NSTKONEWIN );



// Die Hauptschleife: Temperaturen abfragen und Dialog lebendig halten.
for(;;)
	{
	tsnext("main");		// Das Zeitverwaltungssystem tshar lebendig halten.
	}

return 0 ;		// return - will never return !
}


