// $Id$
/** *****************************************************************************************
@doxygen

@file tsonewir.cpp

@brief Demo der Abfragen von Dallas DS1820 One-Wire-Thermometer.
<br>
Siehe auch <a href=../../../deutsch/ts/tsonewir.htm>../../../deutsch/ts/tsonewir.htm</a> - Anwendungsbeschreibung
<br>
<a href=../../../../../etronik/sensor/DS18S20.pdf>../../../../../etronik/sensor/DS18S20.pdf</a> - Der Sensor
<br>
<a href=../../lq/html/tsonewi_8cpp.html>../../lq/html/tsonewi_8cpp.html</a> - Die Unterprogramme

@author HJHerbert

@version 2014-08-16 - 
@version 2014-08-19 - Test und einige kleinere Verbesserungen.

********************************************************************************************* */


#define VERSION "2014-08-19"

#include <heport.h>
#include <tshar.h>
#include <tsser.h>
#include <tsinout.h>
#include <hegettim.h>
#include <tsonewi.h>

// u8 bu[SERNUM] ;

#ifdef AVR
#define NSTKDIA 80 // DIA: Stapel frei =43 verwendet =37 von =80
#else
#define NSTKDIA 2000
#endif

void *stkDIA[NSTKDIA] ;		// fun_dialog


/*!
 * Melde denFehler, dessen PSTR-Text in TSOneWire::fehler
 * eingrtragen war.
 */
FUNCT void Meldefehler(void)
{
if ( TSOneWire::fehler )	// War ein TSOneWire::fehler aufgetreten
	{
	out_puts_P( PSTR(" TSOneWire::fehler:") );
	out_puts_P( TSOneWire::fehler );
	out_nl() ;
	}
}



/*!
 * Version (Datum) anzeigen
 */
FUNCT void out_version(void)
{
out_puts_P( PSTR("\n tsonewir ") ) ;
out_puts_P( PSTR(VERSION) ) ;
}


/*!
 * Hole SERNUM Bytes von einem Sensor
 */
FUNCT void holezeige(u8 *p, u8 n)
{
TSOneWire::HoleNBytes(p,n);
if ( TSOneWire::fehler ) 
	{
	Meldefehler(); 
	return;
	}
// Kein Fehler
out_puts_P( PSTR("\nOK: Zahl in Puffer ist:")  );
for ( u8 i = 0 ; i < n ; ++i )
	{
	out_char(' ');
	out_hex2( p[i] );
	}
}


/*!
 * Anzeigen der Seriennummern
 */
FUNCT void ZeigeROMcodes(void)
{
/*
TSOneWire::Hole8Bytes();
if ( TSOneWire::fehler ) 
	{
	Meldefehler(); 
	return;
	}
*/
// Kein Fehler
for ( u8 k = 0 ; k < TSOneWire_N ; ++k )
	{
	out_puts_P( PSTR("\nKanal") );
	out_uint8( k );
	for ( u8 i = 0 ; i < SERNUMLEN ; ++i )
		{
		out_char(' ');
		out_hex2( TSOneWire::sernum[k][i] );
		}
	}
}


/*!
 * Dialog zum Eingeben der Kanalnummer / Sensornummer 0..3
 */
FUNCT u8 HoleKanal( void )
{
u8 i ;

if ( in_uint8_P( PSTR("Kanal (255=der einzige)"), &i ) != 0 ) return 0xfe ; 
if ( ! ( i == 255 || i < TSOneWire_N ) )
	{
	out_puts_P( PSTR("\nnicht erlaubt") );
	return 0xfe ;
	}
return i ;
}


/*!
 * Alle Seriennummern aus dem EEPROM auslesen
 */
FUNCT void LeseEEPROM(void)
{
out_puts_P( PSTR("\nl - ROMCodes aus dem EEPROM") );
if ( tseeprom_open(			// rv == 0 : OK
			"tsonewir",		// z.B "qqeeprom" wird im AVR ignoriert, unter DOS/Windows ist dies ein Dateiname
    		0x100 )			// z.B: 2048 : wird im AVR ignoriert, unter DOS/Windows ist dies die Gre der Datei
		!= 0 ) return ;
tseeprom_read(
    TSOneWire_EEPROM_ADR,		// Position im EEPROM (oder in der Datei)
	TSOneWire::sernum[0],			// Wohin im Arbeitsspeicher (RAM) die gelesenen Bytes speichern
   	TSOneWire_EEPROM_LNG ) ;		// Wieviele Bytes von EEPROM lesen

tseeprom_close();
}


/*! 
 * Der Dialog durch die serielle Schnittstelle.
 */
FUNCT void fun_dialog(void)
{
char c ;
u8 i ;
u8 j ;
s16 iWert ;

// u8 kom1 ;
// u8 kom2 ;
// Ticks:
/*u32 stt ;		// Startzeit
u32 at ;		// Aktueller Zeit
u32 dt ;		// Differenzzeit
u32 dt1 ;		// Differenzzeit
*/

out_version();
// kom1 = 0xCC ;
// kom2 = 0xBE ;
for(;;)
	{
	while( in_check() == false )
		{		// Warte, bis eine zeichen vom Dialog kommt.
		tsnext("nochar");
		}
	c = in_char() ;	// Dialog: Das Befehlszeichen

	switch ( c )
		{
	case '?' :			// Anzeige der Bedienung
		out_puts_P( PSTR("? - Hilfe anzeigen\n"
		"z - Stackverwendung\n"
		"i - Lese ROM vom einzigen DS1820\n"
		"k - Kopiere ROMCode von Puffer i nach ...\n"
		"e - Eingeben ROMCode in Puffer i\n"
		"r - Zeige ROMCodes im Speicher\n"
		"s - Schreibe ROMCode in EEPROM\n"
		"l - Lese ROMCode von EEPROM\n"
		"t - Temperatur holen\n" ) );

#ifndef AVR
		out_puts_P( PSTR("q - Programm beenden\n") ) ;
#endif
		out_version();

		break ;

#ifndef AVR
	case 'q' :			// Beenden
		exit(0);
#endif

	case 'z' :			// Verwendeten Stapelspeicher anzeigen
		out_puts_P( PSTR("z: \nDIA: ") ); ts_stack_usage( stkDIA, NSTKDIA );	
		break ;

	case 'k' :		// Kopiere ROMCode von LeseROMPuffer i nach ...
		out_puts_P( PSTR("\nk Kopiere DS1820 Seriennummer nach" ));
		i = HoleKanal();
		if ( i >= TSOneWire_N )		// Nicht gueltig
			{
			break ;
			}
		for ( j = 0 ; j < SERNUMLEN ; ++j )
			{
			out_char(' ');
			out_hex2(TSOneWire::sernum[i][j] = TSOneWire::bu[j]);
			}
		break ;

	case 'e' :		// Eingabe ROMCode in den LeseRomPuffer i
		out_puts_P( PSTR("\ne - Eingeben einer DS1820 Seriennummer" ));
		for ( j = 0 ; j < SERNUMLEN ; ++j )
			{
			out_char(' ');
			out_uint8(j);
			out_char(' ');
			if ( in_hex8_P( PSTR(":"), &TSOneWire::bu[j]) != 0 ) break ;
			}
		break ;

	case 'i' :		// Lese ROM - geht nur dann, wenn genau ein DS1820 angeschlossen ist
		out_puts_P( PSTR("\ni - Lese ROM") );

		TSOneWire::Reset();
		if ( TSOneWire::fehler ) { Meldefehler(); break ; } 
//		out_puts_P( PSTR("\nTSOneWire::Reset() erfolgreich") );
		
		TSOneWire::Sende8Bits( 0x33 );	// Lese ROM
		if ( TSOneWire::fehler ) { Meldefehler(); break ; } 

		holezeige(TSOneWire::bus,SERNUMLEN);
		if ( TSOneWire::fehler ) { Meldefehler(); } 
		
		break ;

	case 's':
		out_puts_P( PSTR("\ns - ROMCodes in das EEPROM") );
		if ( tseeprom_open(			// rv == 0 : OK
				    "tsonewir",		// z.B "qqeeprom" wird im AVR ignoriert, unter DOS/Windows ist dies ein Dateiname
    				0x100 )			// z.B: 2048 : wird im AVR ignoriert, unter DOS/Windows ist dies die Gre der Datei
				!= 0 ) break ;
		tseeprom_write(
    		TSOneWire_EEPROM_ADR,             // Position im EEPROM (oder in der Datei)
		    TSOneWire::sernum[0],        // Woher aus dem Arbeitsspeicher (RAM) die zu schreibenden Bytes holen
   			TSOneWire_EEPROM_LNG ) ;             // Wieviele Bytes in das EEPROM speichern

		tseeprom_close();
		break ;

	case 'l':
		LeseEEPROM();
		ZeigeROMcodes();
		break ;
		
	case 'r':
		out_puts_P( PSTR("\nr - Zeige ROMCode im Speicher") );
		ZeigeROMcodes();
		break ;

	case 't' : 		// Lese den Temperaturwert
		out_puts_P( PSTR("\nt - Temperatur ") );
		i = HoleKanal();		// Dialog
		if ( ! ( i == 255 || i < TSOneWire_N ) )
			{
			break ;
			}
		iWert = TSOneWire::Temperatur(i);
		if ( TSOneWire::fehler ) { Meldefehler(); break ; } 

		out_puts_P( PSTR("\nTemperatur:")  );
/* bis 2015-11-25
		out_swfxdp( iWert,				// Wert
			1,							// Anzeige-Nachkommastellen
			16384,						// Normierung: Wert+1 = 0.5 grdC
			0 );						// keine Nachkommastellen in 
*/
		out_swfxdp( iWert,				// Wert
			2,							// Anzeige-Nachkommastellen
			32767,						// Normierung: 32767 = 327.67 grdC
			2 );						// 2 Nachkommastellen in 32767

		out_puts_P( PSTR("grdC")  );
		out_puts_P( PSTR("bu=")  );
		for( i = 0 ; i < BULEN ; ++i )
			{
			out_char(' ');
			out_hex2( TSOneWire::bu[i] );
			}
		out_nl();
		break ;

	case 0x0d:
	case 0x0a:
	case 0x09:
	case 0x20:
		out_version();
		break ;

	default:
		out_puts_P( PSTR("\n* Falsches Kommando. Hilfe=?\n") );
		break ;
		}
	out_puts_P( PSTR("\n") );
	tsnext("fun_dialog");
	}
}


/*!
 * Das Hauptprogramm
 * Hier wird alles eingerichtet und gestartet.
 */
FUNCT int main( int argc, const char **argv )
{
tsseropen();

ts_stack_fill( stkDIA, NSTKDIA );	// Muster im STACK ablegen fuer Messung der Belegung

tickopen();
tsopen();		// Tabellen initialisieren, 


/*
vout_char[1] = tsser_out_char ; 
vin_char[1] = tsser_in_char ;
vin_check[1] = tsser_in_check ;

	vout_char[i] = tsser_out_char ; 
	vin_char[i] = tsser_in_char ;
	vin_check[i] = tsser_in_check ;

*/

tseinri( 1 , fun_dialog, "fun_dialog", stkDIA, sizeof(stkDIA) );

tsnext( "main" );	// run one piece of fun_dialog

for(;;)
	{
	tsnext("main");
	}

return 0 ;		// return - will never return !
}


